

CREATE OR REPLACE FUNCTION spCreateFA_Depr(
	_FAID			VARCHAR,
	_DeprID			VARCHAR,
	_MaxDeprMonth		INT,
	_TotalDeprMonth	INT,
	_EcLife			NUMERIC(19,4),
	_MaxAccumCost		NUMERIC(19,4),
	_AcqDate		DATE,
	_AcqCost		NUMERIC(19,4),
	_RsdValue		NUMERIC(19,4),
	_PerStart		DATE,
	_UsageDate		DATE,
	_DoGenJnlDepr		BOOLEAN
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
BEGIN
	/** Inisialisasi */

	SELECT INTO _spret * FROM spCreateFA_Depr(_FAID, _DeprID, _MaxDeprMonth, _TotalDeprMonth, _EcLife, 
		_MaxAccumCost, _AcqDate, _AcqCost, _RsdValue, _PerStart, _UsageDate, _DoGenJnlDepr, FALSE);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCreateFA_Depr(
	_FAID			VARCHAR,
	_DeprID			VARCHAR,
	_MaxDeprMonth		INT,
	__TotalDeprMonth	INT,
	_EcLife			NUMERIC(19,4),
	_MaxAccumCost		NUMERIC(19,4),
	_AcqDate		DATE,
	_AcqCost		NUMERIC(19,4),
	_RsdValue		NUMERIC(19,4),
	_PerStart		DATE,
	_UsageDate		DATE,
	_DoGenJnlDepr		BOOLEAN,
	_DoGenJnlBegBal		BOOLEAN
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_AkumulasiPerTahun	NUMERIC(19,4);
	_AkumulasiPerBulan	NUMERIC(19,4);
	_SelisihPerTahun	NUMERIC(19,4);
	_SelisihPerLife		NUMERIC(19,4);

	_PersenPerTahun		NUMERIC(19,4);
	_YearCounter		INT;
	_YearRemain		INT;
	_OldYearCounter		INT;

	_DeprAmt		NUMERIC(19,4);
	_TotalDeprAmt		NUMERIC(19,4);

	_LastDeprMonth		INT;	

	_MonthIncrement		INT;

	_FADeprNo		VARCHAR;
	_DeprDate		DATE;
	
	_LockDate		DATE;

	_AccHist		VARCHAR;
	_AccAccum		VARCHAR;
	_AccAsset		VARCHAR;
	_FAName			VARCHAR;
	_FABV			NUMERIC(19,4);
	_BranchID		VARCHAR;
	_DeptID			VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
	_JnlDNo			INT2;

	_TotalDeprMonth		INT;
	_Depreciatable		BOOLEAN;
BEGIN
	/** Inisialisasi */
	_AkumulasiPerTahun:=0;
	_AkumulasiPerBulan:=0;
	_SelisihPerTahun:=0;
	_SelisihPerLife:=0;
	_DeprAmt:=0;
	_TotalDeprAmt:=0;
	_LastDeprMonth:=0;
	_YearCounter:=0;
	_OldYearCounter:=-1;
	_TotalDeprMonth:=__TotalDeprMonth;
	
	SELECT INTO _LastDeprMonth COALESCE(CAST(MAX(accummonth) AS INT), 0) 
	FROM fadepr WHERE faid=_FAID;

	SELECT INTO _AccAsset, _AccAccum, _FAName, _BranchID, _DeptID, _FABV, _Depreciatable 
		fa.accasset, fa.accaccum, fa.faname, fa.branchid, fa.deptid, fa.fabv, g.depreciatable 
	FROM fa 
	JOIN fagrp g ON fa.fagrpid=g.fagrpid
	WHERE faid=_FAID; 

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;


	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='JNL';
	IF NOT FOUND THEN
		_LockDate:=NULL;
	END IF;

	IF (_TotalDeprMonth>_MaxDeprMonth) THEN
		_TotalDeprMonth:=_MaxDeprMonth;
	END IF;

	_MonthIncrement:=_TotalDeprMonth-_LastDeprMonth;

	IF (_Depreciatable=TRUE AND _DeprID='SLM') THEN /** Metode garis lurus */
		_AkumulasiPerTahun:=ROUND(_MaxAccumCost/_EcLife);
		_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
		_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
		_SelisihPerLife:=_MaxAccumCost-(_AkumulasiPerTahun*_EcLife);

		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _UsageDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Tanggal Pembelian Melebihi LockDate! Mundurkan LockDate Terlebih Dahulu!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			
			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;
			INSERT INTO fadepr(fadeprno, faid, fadeprdate, depramt, accummonth) 
					VALUES(_FADeprNo, _FAID, _DeprDate, _DeprAmt, _LastDeprMonth);

			SELECT INTO _spret * FROM spFixedAsset_Depr_New(_FADeprNo, _DoGenJnlDepr);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;

		UPDATE fa SET accumytd=(CASE WHEN active=TRUE THEN _AkumulasiPerTahun ELSE 0 END) WHERE faid=_FAID;
	ELSIF (_Depreciatable=TRUE AND _DeprID='DBM') THEN /** Metode saldo menurun */
		_PersenPerTahun:=(100/_EcLife)/100;
		_AkumulasiPerTahun:=_MaxAccumCost*_PersenPerTahun;

		IF (_MaxAccumCost=0) THEN
			_PersenPerTahun:=0;
		ELSE 
			_PersenPerTahun:=(_AkumulasiPerTahun/_MaxAccumCost)*2;
		END IF;

		IF (_PersenPerTahun>0.5) THEN
			_PersenPerTahun:=0.5;
		END IF;
		
		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_YearCounter:=(_LastDeprMonth/12);
			_YearRemain:=MOD(_LastDeprMonth, 12);
			
			IF ((_YearCounter>_OldYearCounter AND _YearRemain<>0) OR (_YearCounter=_OldYearCounter AND _YearRemain=1)) THEN
				_OldYearCounter:=_YearCounter;

				IF (_YearCounter<(_EcLife-1)) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=(_EcLife-1)) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;	
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;		
				END IF;

				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			ELSIF (_YearCounter>_OldYearCounter AND _YearRemain=0) THEN
				_OldYearCounter:=_YearCounter;
				
				IF (_YearCounter<_EcLife) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=36) THEN
--						RAISE EXCEPTION 'GNP 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=_EcLife) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;
--					IF (_LastDeprMonth=48) THEN
--						RAISE EXCEPTION 'GNP 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;					
				END IF;				
				
				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			END IF;			
			
--			IF (_LastDeprMonth=48) THEN
--				RAISE EXCEPTION '0 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--			END IF;

			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _UsageDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Tanggal Pembelian Melebihi LockDate! Mundurkan LockDate Terlebih Dahulu!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;
			INSERT INTO fadepr(fadeprno, faid, fadeprdate, depramt, accummonth) 
					VALUES(_FADeprNo, _FAID, _DeprDate, _DeprAmt, _LastDeprMonth);

			SELECT INTO _spret * FROM spFixedAsset_Depr_New(_FADeprNo, _DoGenJnlDepr);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;

		UPDATE fa SET accumytd=(CASE WHEN active=TRUE THEN _AkumulasiPerTahun ELSE 0 END) WHERE faid=_FAID;
	ELSIF (_Depreciatable=FALSE) THEN
		_TotalDeprAmt:=0;
	END IF;

	IF (isAccInstalled() AND _DoGenJnlBegBal=TRUE) THEN
		_AccHist:=fAccDefault('BEGBAL');
		IF (_AccHist IS NULL OR _AccHist='') THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Default Akun Saldo Awal belum diset dalam sistem (spCreateFA_Depr)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:='Saldo Awal Harta Tetap '||_FAName;
			
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'BEG', _PerStart,'FA', _FAID,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			null, null, null, null,
			null, _BranchID, _DeptID, null, _CrcID, _ExcRate, _FisRate				
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_JnlDNo:=1;
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccAsset, 'D', _AcqCost, _DeptID, null, _AcqCost);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_TotalDeprAmt!=0) THEN
			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccAccum, 'D', (_TotalDeprAmt*(-1)), _DeptID, null, (_TotalDeprAmt*(-1)));
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		_JnlDNo:=_JnlDNo+1;
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccHist, 'K', (_AcqCost-_TotalDeprAmt), _DeptID, null, (_AcqCost-_TotalDeprAmt));
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


